@@ -2,7 +2,6 @@ package operator
 
 import (
 	"context"
-	"fmt"
 	"os"
 	"time"
 
@@ -15,16 +14,12 @@ import (
 	operatorv1client "github.com/openshift/client-go/operator/clientset/versioned"
 	operatorv1informers "github.com/openshift/client-go/operator/informers/externalversions"
 	"github.com/openshift/library-go/pkg/controller/controllercmd"
-	"github.com/openshift/library-go/pkg/operator/encryption"
 	"github.com/openshift/library-go/pkg/operator/encryption/controllers/migrators"
 	encryptiondeployer "github.com/openshift/library-go/pkg/operator/encryption/deployer"
 	"github.com/openshift/library-go/pkg/operator/events"
 	"github.com/openshift/library-go/pkg/operator/genericoperatorclient"
-	"github.com/openshift/library-go/pkg/operator/resource/resourceapply"
-	"github.com/openshift/library-go/pkg/operator/revisioncontroller"
 	"github.com/openshift/library-go/pkg/operator/staleconditions"
 	"github.com/openshift/library-go/pkg/operator/staticpod/controller/revision"
-	"github.com/openshift/library-go/pkg/operator/staticresourcecontroller"
 	"github.com/openshift/library-go/pkg/operator/status"
 	"github.com/openshift/library-go/pkg/operator/v1helpers"
 	"k8s.io/apimachinery/pkg/api/errors"
@@ -39,15 +34,18 @@ import (
 	apiregistrationinformers "k8s.io/kube-aggregator/pkg/client/informers/externalversions"
 	utilpointer "k8s.io/utils/pointer"
 
-	"github.com/openshift/cluster-openshift-apiserver-operator/pkg/operator/apiservercontrollerset"
-	"github.com/openshift/cluster-openshift-apiserver-operator/pkg/operator/apiservicecontroller"
+	"github.com/openshift/cluster-openshift-apiserver-operator/pkg/operator/apiservice"
 	"github.com/openshift/cluster-openshift-apiserver-operator/pkg/operator/configobservation/configobservercontroller"
 	"github.com/openshift/cluster-openshift-apiserver-operator/pkg/operator/configobservation/etcdobserver"
+	"github.com/openshift/cluster-openshift-apiserver-operator/pkg/operator/encryptionprovider"
+	"github.com/openshift/cluster-openshift-apiserver-operator/pkg/operator/oauthapiencryptioncontroller"
 	"github.com/openshift/cluster-openshift-apiserver-operator/pkg/operator/operatorclient"
 	prune "github.com/openshift/cluster-openshift-apiserver-operator/pkg/operator/prunecontroller"
 	"github.com/openshift/cluster-openshift-apiserver-operator/pkg/operator/resourcesynccontroller"
 	"github.com/openshift/cluster-openshift-apiserver-operator/pkg/operator/v311_00_assets"
-	"github.com/openshift/cluster-openshift-apiserver-operator/pkg/operator/workloadcontroller"
+	operatorworkload "github.com/openshift/cluster-openshift-apiserver-operator/pkg/operator/workload"
+	workloadcontroller "github.com/openshift/library-go/pkg/operator/apiserver/controller/workload"
+	apiservercontrollerset "github.com/openshift/library-go/pkg/operator/apiserver/controllerset"
 )
 
 func RunOperator(ctx context.Context, controllerConfig *controllercmd.ControllerContext) error {
@@ -76,6 +74,7 @@ func RunOperator(ctx context.Context, controllerConfig *controllercmd.Controller
 		operatorclient.OperatorNamespace,
 		operatorclient.TargetNamespace,
 		etcdobserver.EtcdEndpointNamespace,
+		metav1.NamespaceSystem,
 	)
 	apiregistrationInformers := apiregistrationinformers.NewSharedInformerFactory(apiregistrationv1Client, 10*time.Minute)
 	configInformers := configinformers.NewSharedInformerFactory(configClient, 10*time.Minute)
@@ -85,21 +84,6 @@ func RunOperator(ctx context.Context, controllerConfig *controllercmd.Controller
 		return err
 	}
 
-	staticResourceController := staticresourcecontroller.NewStaticResourceController(
-		"OpenshiftAPIServerStaticResources",
-		v311_00_assets.Asset,
-		[]string{
-			"v3.11.0/openshift-apiserver/ns.yaml",
-			"v3.11.0/openshift-apiserver/apiserver-clusterrolebinding.yaml",
-			"v3.11.0/openshift-apiserver/svc.yaml",
-			"v3.11.0/openshift-apiserver/sa.yaml",
-			"v3.11.0/openshift-apiserver/trusted_ca_cm.yaml",
-		},
-		resourceapply.NewKubeClientHolder(kubeClient),
-		operatorClient,
-		controllerConfig.EventRecorder,
-	).AddKubeInformers(kubeInformersForNamespaces)
-
 	resourceSyncController, debugHandler, err := resourcesynccontroller.NewResourceSyncController(
 		operatorClient,
 		kubeInformersForNamespaces,
@@ -111,23 +95,9 @@ func RunOperator(ctx context.Context, controllerConfig *controllercmd.Controller
 		return err
 	}
 
-	revisionController := revisioncontroller.NewRevisionController(
-		operatorclient.TargetNamespace,
-		nil,
-		[]revision.RevisionResource{{
-			Name:     "encryption-config",
-			Optional: true,
-		}},
-		kubeInformersForNamespaces.InformersFor(operatorclient.TargetNamespace),
-		OpenshiftDeploymentLatestRevisionClient{OperatorClient: operatorClient, TypedClient: operatorConfigClient.OperatorV1()},
-		v1helpers.CachedConfigMapGetter(kubeClient.CoreV1(), kubeInformersForNamespaces),
-		v1helpers.CachedSecretGetter(kubeClient.CoreV1(), kubeInformersForNamespaces),
-		controllerConfig.EventRecorder,
-	)
-
 	// don't change any versions until we sync
 	versionRecorder := status.NewVersionGetter()
-	clusterOperator, err := configClient.ConfigV1().ClusterOperators().Get("openshift-apiserver", metav1.GetOptions{})
+	clusterOperator, err := configClient.ConfigV1().ClusterOperators().Get(ctx, "openshift-apiserver", metav1.GetOptions{})
 	if err != nil && !errors.IsNotFound(err) {
 		return err
 	}
@@ -136,32 +106,47 @@ func RunOperator(ctx context.Context, controllerConfig *controllercmd.Controller
 	}
 	versionRecorder.SetVersion("operator", os.Getenv("OPERATOR_IMAGE_VERSION"))
 
-	nodeInformer := kubeInformersForNamespaces.InformersFor("").Core().V1().Nodes()
+	nodeProvider := encryptiondeployer.NewDeploymentNodeProvider(operatorclient.TargetNamespace, kubeInformersForNamespaces)
+	deployer, err := encryptiondeployer.NewRevisionLabelPodDeployer("revision", operatorclient.TargetNamespace, kubeInformersForNamespaces, resourceSyncController, kubeClient.CoreV1(), kubeClient.CoreV1(), nodeProvider)
+	if err != nil {
+		return err
+	}
 
-	workloadController := workloadcontroller.NewWorkloadController(
-		os.Getenv("IMAGE"), os.Getenv("OPERATOR_IMAGE_VERSION"), os.Getenv("OPERATOR_IMAGE"),
+	migrationClient := kubemigratorclient.NewForConfigOrDie(controllerConfig.KubeConfig)
+	migrationInformer := migrationv1alpha1informer.NewSharedInformerFactory(migrationClient, time.Minute*30)
+	migrator := migrators.NewKubeStorageVersionMigrator(migrationClient, migrationInformer.Migration().V1alpha1(), kubeClient.Discovery())
+	encryptionProvider := encryptionprovider.New(
+		"openshift-oauth-apiserver",
+		oauthapiencryptioncontroller.EncryptionConfigManagedBy,
+		[]schema.GroupResource{
+			{Group: "route.openshift.io", Resource: "routes"}, // routes can contain embedded TLS private keys
+			{Group: "oauth.openshift.io", Resource: "oauthaccesstokens"},
+			{Group: "oauth.openshift.io", Resource: "oauthauthorizetokens"},
+		},
+		sets.NewString("oauthaccesstokens.oauth.openshift.io", "oauthauthorizetokens.oauth.openshift.io"),
+		kubeInformersForNamespaces,
+	)
+
+	openShiftAPIServerWorkload := operatorworkload.NewOpenShiftAPIServerWorkload(
 		operatorClient,
-		versionRecorder,
-		operatorConfigInformers.Operator().V1().OpenShiftAPIServers(),
-		kubeInformersForNamespaces.InformersFor(operatorclient.TargetNamespace),
-		kubeInformersForNamespaces.InformersFor(operatorclient.GlobalUserSpecifiedConfigNamespace),
-		kubeInformersForNamespaces.InformersFor(operatorclient.GlobalUserSpecifiedConfigNamespace),
-		apiregistrationInformers,
-		configInformers,
-		nodeInformer,
 		operatorConfigClient.OperatorV1(),
 		configClient.ConfigV1(),
+		workloadcontroller.CountNodesFuncWrapper(kubeInformersForNamespaces.InformersFor("").Core().V1().Nodes().Lister()),
+		workloadcontroller.EnsureAtMostOnePodPerNode,
+		"openshift-apiserver",
+		os.Getenv("IMAGE"),
+		os.Getenv("OPERATOR_IMAGE"),
 		kubeClient,
 		controllerConfig.EventRecorder,
-	)
+		versionRecorder)
 
 	apiServerControllers := apiservercontrollerset.NewAPIServerControllerSet(
 		operatorClient,
 		controllerConfig.EventRecorder,
 	).WithAPIServiceController(
 		"openshift-apiserver",
-		apiservicecontroller.NewAPIServicesToManage(
-			apiregistrationv1Client.ApiregistrationV1(),
+		apiservice.NewAPIServicesToManage(
+			apiregistrationInformers.Apiregistration().V1().APIServices().Lister(),
 			operatorConfigInformers.Operator().V1().Authentications().Lister(),
 			apiServices(),
 			controllerConfig.EventRecorder,
@@ -185,6 +170,7 @@ func RunOperator(ctx context.Context, controllerConfig *controllercmd.Controller
 				{Resource: "namespaces", Name: operatorclient.GlobalMachineSpecifiedConfigNamespace},
 				{Resource: "namespaces", Name: operatorclient.OperatorNamespace},
 				{Resource: "namespaces", Name: operatorclient.TargetNamespace},
+				{Resource: "namespaces", Name: "openshift-etcd-operator"}, // Capture events from etcd operator
 				{Resource: "endpoints", Name: etcdobserver.EtcdEndpointName, Namespace: etcdobserver.EtcdEndpointNamespace},
 			},
 			apiServicesReferences()...,
@@ -192,6 +178,52 @@ func RunOperator(ctx context.Context, controllerConfig *controllercmd.Controller
 		configClient.ConfigV1(),
 		configInformers.Config().V1().ClusterOperators(),
 		versionRecorder,
+	).WithWorkloadController(
+		"OpenShiftAPIServer",
+		operatorclient.OperatorNamespace,
+		operatorclient.TargetNamespace,
+		os.Getenv("OPERATOR_IMAGE_VERSION"),
+		"openshift",
+		"APIServer",
+		kubeClient,
+		openShiftAPIServerWorkload,
+		configClient.ConfigV1().ClusterOperators(),
+		versionRecorder,
+		kubeInformersForNamespaces,
+		operatorConfigInformers.Operator().V1().OpenShiftAPIServers().Informer(),
+		configInformers.Config().V1().Images().Informer(),
+	).WithStaticResourcesController(
+		"APIServerStaticResources",
+		v311_00_assets.Asset,
+		[]string{
+			"v3.11.0/openshift-apiserver/ns.yaml",
+			"v3.11.0/openshift-apiserver/apiserver-clusterrolebinding.yaml",
+			"v3.11.0/openshift-apiserver/svc.yaml",
+			"v3.11.0/openshift-apiserver/sa.yaml",
+			"v3.11.0/openshift-apiserver/trusted_ca_cm.yaml",
+		},
+		kubeInformersForNamespaces,
+		kubeClient,
+	).WithRevisionController(
+		operatorclient.TargetNamespace,
+		nil,
+		[]revision.RevisionResource{{
+			Name:     "encryption-config",
+			Optional: true,
+		}},
+		kubeInformersForNamespaces.InformersFor(operatorclient.TargetNamespace),
+		OpenshiftDeploymentLatestRevisionClient{OperatorClient: operatorClient, TypedClient: operatorConfigClient.OperatorV1()},
+		v1helpers.CachedConfigMapGetter(kubeClient.CoreV1(), kubeInformersForNamespaces),
+		v1helpers.CachedSecretGetter(kubeClient.CoreV1(), kubeInformersForNamespaces),
+	).WithEncryptionControllers(
+		operatorclient.TargetNamespace,
+		encryptionProvider,
+		deployer,
+		migrator,
+		kubeClient.CoreV1(),
+		configClient.ConfigV1().APIServers(),
+		configInformers.Config().V1().APIServers(),
+		kubeInformersForNamespaces,
 	).WithConfigUpgradableController().
 		WithLogLevelController()
 
@@ -210,38 +242,6 @@ func RunOperator(ctx context.Context, controllerConfig *controllercmd.Controller
 		controllerConfig.EventRecorder,
 	)
 
-	nodeProvider := DeploymentNodeProvider{
-		TargetNamespaceDeploymentInformer: kubeInformersForNamespaces.InformersFor(operatorclient.TargetNamespace).Apps().V1().Deployments(),
-		NodeInformer:                      nodeInformer,
-	}
-
-	deployer, err := encryptiondeployer.NewRevisionLabelPodDeployer("revision", operatorclient.TargetNamespace, kubeInformersForNamespaces, resourceSyncController, kubeClient.CoreV1(), kubeClient.CoreV1(), nodeProvider)
-	if err != nil {
-		return err
-	}
-
-	migrationClient := kubemigratorclient.NewForConfigOrDie(controllerConfig.KubeConfig)
-	migrationInformer := migrationv1alpha1informer.NewSharedInformerFactory(migrationClient, time.Minute*30)
-	migrator := migrators.NewKubeStorageVersionMigrator(migrationClient, migrationInformer.Migration().V1alpha1(), kubeClient.Discovery())
-
-	encryptionControllers, err := encryption.NewControllers(
-		operatorclient.TargetNamespace,
-		deployer,
-		migrator,
-		operatorClient,
-		configClient.ConfigV1().APIServers(),
-		configInformers.Config().V1().APIServers(),
-		kubeInformersForNamespaces,
-		kubeClient.CoreV1(),
-		controllerConfig.EventRecorder,
-		schema.GroupResource{Group: "route.openshift.io", Resource: "routes"}, // routes can contain embedded TLS private keys
-		schema.GroupResource{Group: "oauth.openshift.io", Resource: "oauthaccesstokens"},
-		schema.GroupResource{Group: "oauth.openshift.io", Resource: "oauthauthorizetokens"},
-	)
-	if err != nil {
-		return err
-	}
-
 	pruneController := prune.NewPruneController(
 		operatorclient.TargetNamespace,
 		[]string{"encryption-config-"},
@@ -251,7 +251,7 @@ func RunOperator(ctx context.Context, controllerConfig *controllercmd.Controller
 		controllerConfig.EventRecorder,
 	)
 
-	staleConditions := staleconditions.NewRemoveStaleConditions(
+	staleConditions := staleconditions.NewRemoveStaleConditionsController(
 		[]string{
 			// in 4.1.0-4.3.0 this was used for indicating the apiserver daemonset was progressing
 			"Progressing",
@@ -261,6 +261,9 @@ func RunOperator(ctx context.Context, controllerConfig *controllercmd.Controller
 			"APIServerDaemonSetAvailable",
 			"APIServerDaemonSetProgressing",
 			"APIServerDaemonSetDegraded",
+			// in 4.5.z we changed (renamed) the following conditions
+			"OpenshiftAPIServerStaticResourcesDegraded", // became APIServerStaticResourcesDegraded
+			"WorkloadDegraded",                          // became APIServerWorkloadDegraded
 		},
 		operatorClient,
 		controllerConfig.EventRecorder,
@@ -270,7 +273,7 @@ func RunOperator(ctx context.Context, controllerConfig *controllercmd.Controller
 		controllerConfig.Server.Handler.NonGoRestfulMux.Handle("/debug/controllers/resourcesync", debugHandler)
 	}
 
-	ensureDaemonSetCleanup(kubeClient, ctx, controllerConfig.EventRecorder)
+	ensureDaemonSetCleanup(ctx, kubeClient, controllerConfig.EventRecorder)
 
 	operatorConfigInformers.Start(ctx.Done())
 	kubeInformersForNamespaces.Start(ctx.Done())
@@ -279,18 +282,14 @@ func RunOperator(ctx context.Context, controllerConfig *controllercmd.Controller
 	dynamicInformers.Start(ctx.Done())
 	migrationInformer.Start(ctx.Done())
 
-	go staticResourceController.Run(ctx, 1)
-	go workloadController.Run(ctx, 1)
 	go configObserver.Run(ctx, 1)
 	go resourceSyncController.Run(ctx, 1)
-	go revisionController.Run(ctx, 1)
-	go encryptionControllers.Run(ctx.Done())
-	go pruneController.Run(ctx)
+	go pruneController.Run(ctx, 1)
 	go runnableAPIServerControllers.Run(ctx)
 	go staleConditions.Run(ctx, 1)
 
 	<-ctx.Done()
-	return fmt.Errorf("stopped")
+	return nil
 }
 
 func apiServices() []*apiregistrationv1.APIService {
@@ -348,7 +347,7 @@ func apiServicesReferences() []configv1.ObjectReference {
 // used to manage apiserver pods in releases prior to 4.5. The daemonset is
 // removed once the deployment now managing apiserver pods reports at least
 // one pod available.
-func ensureDaemonSetCleanup(kubeClient *kubernetes.Clientset, ctx context.Context, eventRecorder events.Recorder) {
+func ensureDaemonSetCleanup(ctx context.Context, kubeClient *kubernetes.Clientset, eventRecorder events.Recorder) {
 	// daemonset and deployment both use the same name
 	resourceName := "apiserver"
 
@@ -361,7 +360,7 @@ func ensureDaemonSetCleanup(kubeClient *kubernetes.Clientset, ctx context.Contex
 		// checking.
 
 		// Check whether the legacy daemonset exists and is not marked for deletion
-		ds, err := dsClient.Get(resourceName, metav1.GetOptions{})
+		ds, err := dsClient.Get(ctx, resourceName, metav1.GetOptions{})
 		if errors.IsNotFound(err) {
 			// Done - daemonset does not exist
 			return
@@ -376,7 +375,7 @@ func ensureDaemonSetCleanup(kubeClient *kubernetes.Clientset, ctx context.Contex
 		}
 
 		// Check that the deployment managing the apiserver pods has at last one available replica
-		deploy, err := deployClient.Get(resourceName, metav1.GetOptions{})
+		deploy, err := deployClient.Get(ctx, resourceName, metav1.GetOptions{})
 		if errors.IsNotFound(err) {
 			// No available replicas if the deployment doesn't exist
 			return
@@ -391,7 +390,7 @@ func ensureDaemonSetCleanup(kubeClient *kubernetes.Clientset, ctx context.Contex
 		}
 
 		// Safe to remove legacy daemonset since the deployment has at least one available replica
-		err = dsClient.Delete(resourceName, &metav1.DeleteOptions{})
+		err = dsClient.Delete(ctx, resourceName, metav1.DeleteOptions{})
 		if err != nil && !errors.IsNotFound(err) {
 			klog.Warningf("Failed to delete legacy daemonset: %v", err)
 			return
